@@ -2,6 +2,7 @@
 #define PG_QUERY_H
 
 #include <stdint.h>
+#include <sys/types.h>
 
 typedef struct {
 	char* message; // exception message
@@ -13,7 +14,7 @@ typedef struct {
 } PgQueryError;
 
 typedef struct {
-  unsigned int len;
+  size_t len;
   char* data;
 } PgQueryProtobuf;
 
@@ -69,17 +70,42 @@ typedef struct {
   PgQueryError* error;
 } PgQueryNormalizeResult;
 
+// Postgres parser options (parse mode and GUCs that affect parsing)
+
+typedef enum
+{
+	PG_QUERY_PARSE_DEFAULT = 0,
+	PG_QUERY_PARSE_TYPE_NAME,
+	PG_QUERY_PARSE_PLPGSQL_EXPR,
+	PG_QUERY_PARSE_PLPGSQL_ASSIGN1,
+	PG_QUERY_PARSE_PLPGSQL_ASSIGN2,
+	PG_QUERY_PARSE_PLPGSQL_ASSIGN3
+} PgQueryParseMode;
+
+// We technically only need 3 bits to store parse mode, but
+// having 4 bits avoids API breaks if another one gets added.
+#define PG_QUERY_PARSE_MODE_BITS 4
+#define PG_QUERY_PARSE_MODE_BITMASK ((1 << PG_QUERY_PARSE_MODE_BITS) - 1)
+
+#define PG_QUERY_DISABLE_BACKSLASH_QUOTE 16 // backslash_quote = off (default is safe_encoding, which is effectively on)
+#define PG_QUERY_DISABLE_STANDARD_CONFORMING_STRINGS 32 // standard_conforming_strings = off (default is on)
+#define PG_QUERY_DISABLE_ESCAPE_STRING_WARNING 64 // escape_string_warning = off (default is on)
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 PgQueryNormalizeResult pg_query_normalize(const char* input);
+PgQueryNormalizeResult pg_query_normalize_utility(const char* input);
 PgQueryScanResult pg_query_scan(const char* input);
 PgQueryParseResult pg_query_parse(const char* input);
+PgQueryParseResult pg_query_parse_opts(const char* input, int parser_options);
 PgQueryProtobufParseResult pg_query_parse_protobuf(const char* input);
+PgQueryProtobufParseResult pg_query_parse_protobuf_opts(const char* input, int parser_options);
 PgQueryPlpgsqlParseResult pg_query_parse_plpgsql(const char* input);
 
 PgQueryFingerprintResult pg_query_fingerprint(const char* input);
+PgQueryFingerprintResult pg_query_fingerprint_opts(const char* input, int parser_options);
 
 // Use pg_query_split_with_scanner when you need to split statements that may
 // contain parse errors, otherwise pg_query_split_with_parser is recommended
@@ -106,9 +132,9 @@ void pg_query_free_fingerprint_result(PgQueryFingerprintResult result);
 void pg_query_exit(void);
 
 // Postgres version information
-#define PG_MAJORVERSION "15"
-#define PG_VERSION "15.1"
-#define PG_VERSION_NUM 150001
+#define PG_MAJORVERSION "17"
+#define PG_VERSION "17.0"
+#define PG_VERSION_NUM 170000
 
 // Deprecated APIs below
 
